@@ -36,6 +36,7 @@ KITINERARY_MAKE_SIMPLE_CLASS(Seat)
 KITINERARY_MAKE_PROPERTY(Seat, QString, seatNumber, setSeatNumber)
 KITINERARY_MAKE_PROPERTY(Seat, QString, seatRow, setSeatRow)
 KITINERARY_MAKE_PROPERTY(Seat, QString, seatSection, setSeatSection)
+KITINERARY_MAKE_OPERATOR(Seat)
 
 class TicketPrivate : public QSharedData
 {
@@ -47,6 +48,7 @@ public:
 KITINERARY_MAKE_SIMPLE_CLASS(Ticket)
 KITINERARY_MAKE_PROPERTY(Ticket, Seat, ticketedSeat, setTicketedSeat)
 KITINERARY_MAKE_PROPERTY(Ticket, QString, ticketToken, setTicketToken)
+KITINERARY_MAKE_OPERATOR(Ticket)
 
 Ticket::TicketTokenType Ticket::ticketTokenType() const
 {
@@ -54,6 +56,8 @@ Ticket::TicketTokenType Ticket::ticketTokenType() const
         return QRCode;
     } else if (d->ticketToken.startsWith(QLatin1String("aztec"), Qt::CaseInsensitive)) {
         return AztecCode;
+    } else if (d->ticketToken.startsWith(QLatin1String("barcode128:"), Qt::CaseInsensitive)) {
+        return Code128;
     } else if (d->ticketToken.startsWith(QLatin1String("http"), Qt::CaseInsensitive)) {
         return Url;
     }
@@ -69,6 +73,8 @@ QString Ticket::ticketTokenData() const
     } else if (d->ticketToken.startsWith(QLatin1String("aztecbin:"), Qt::CaseInsensitive)) {
         const auto b = QByteArray::fromBase64(d->ticketToken.midRef(9).toLatin1());
         return QString::fromLatin1(b.constData(), b.size());
+    } else if (d->ticketToken.startsWith(QLatin1String("barcode128:"), Qt::CaseInsensitive)) {
+        return ticketToken().mid(11);
     }
     return ticketToken();
 }
