@@ -29,19 +29,19 @@
 #include <KItinerary/Action>
 #include <KItinerary/AirportDb>
 #include <KItinerary/BusTrip>
+#include <KItinerary/Event>
 #include <KItinerary/Flight>
 #include <KItinerary/Organization>
 #include <KItinerary/Person>
 #include <KItinerary/Place>
+#include <KItinerary/RentalCar>
 #include <KItinerary/Reservation>
 #include <KItinerary/Ticket>
 #include <KItinerary/TrainTrip>
 #include <KItinerary/TrainStationDb>
 #include <KItinerary/Visit>
 
-#ifdef HAVE_KCONTACTS
 #include <KContacts/Address>
-#endif
 
 #include <QDebug>
 #include <QJsonArray>
@@ -75,18 +75,27 @@ public:
     LodgingReservation processLodgingReservation(LodgingReservation res) const;
     FoodEstablishmentReservation processFoodEstablishmentReservation(FoodEstablishmentReservation res) const;
     TouristAttractionVisit processTouristAttractionVisit(TouristAttractionVisit visit) const;
+    EventReservation processEventReservation(EventReservation res) const;
+    RentalCarReservation processRentalCarReservation(RentalCarReservation res) const;
+    RentalCar processRentalCar(RentalCar car) const;
+    TaxiReservation processTaxiReservation(TaxiReservation res) const;
+    Event processEvent(Event event) const;
 
     template <typename T> T processReservation(T res) const;
     Person processPerson(Person person) const;
     template <typename T> T processPlace(T place) const;
     QVariantList processActions(QVariantList actions) const;
+    template <typename T> QDateTime processTimeForLocation(QDateTime dt, const T &place) const;
 
     bool filterReservation(const QVariant &res) const;
     bool filterLodgingReservation(const LodgingReservation &res) const;
     bool filterFlight(const Flight &flight) const;
     bool filterAirport(const Airport &airport) const;
-    template <typename T> bool filterTrainOrBusTrip(const T &trip) const;
+    bool filterTrainTrip(const TrainTrip &trip) const;
+    bool filterBusTrip(const BusTrip &trip) const;
     template <typename T> bool filterTrainOrBusStation(const T &station) const;
+    bool filterEventReservation(const EventReservation &res) const;
+    bool filterFoodReservation(const FoodEstablishmentReservation &res) const;
 
     QVector<QVariant> m_data;
     QDateTime m_contextDate;
@@ -119,6 +128,12 @@ void ExtractorPostprocessor::process(const QVector<QVariant> &data)
             elem = d->processTouristAttractionVisit(elem.value<TouristAttractionVisit>());
         } else if (JsonLd::isA<BusReservation>(elem)) {
             elem = d->processBusReservation(elem.value<BusReservation>());
+        } else if (JsonLd::isA<EventReservation>(elem)) {
+            elem = d->processEventReservation(elem.value<EventReservation>());
+        } else if (JsonLd::isA<RentalCarReservation>(elem)) {
+            elem = d->processRentalCarReservation(elem.value<RentalCarReservation>());
+        } else if (JsonLd::isA<TaxiReservation>(elem)) {
+            elem = d->processTaxiReservation(elem.value<TaxiReservation>());
         }
 
         d->mergeOrAppend(elem);
@@ -157,7 +172,7 @@ void ExtractorPostprocessorPrivate::mergeOrAppend(const QVariant &elem)
     if (it == m_data.end()) {
         m_data.push_back(elem);
     } else {
-        *it = JsonLdDocument::apply(*it, elem);
+        *it = MergeUtil::merge(*it, elem);
     }
 }
 
@@ -197,7 +212,7 @@ Flight ExtractorPostprocessorPrivate::processFlight(Flight flight) const
 Airport ExtractorPostprocessorPrivate::processAirport(Airport airport) const
 {
     // clean up name
-    airport.setName(airport.name().trimmed());
+    airport.setName(airport.name().simplified());
 
     // complete missing IATA codes
     auto iataCode = airport.iataCode();
@@ -337,7 +352,7 @@ QDateTime ExtractorPostprocessorPrivate::processTrainTripTime(QDateTime dt, cons
         return dt;
     }
 
-    if (dt.timeSpec() == Qt::TimeZone || station.identifier().isEmpty()) {
+    if (dt.timeSpec() == Qt::TimeZone) {
         return dt;
     }
 
@@ -348,6 +363,8 @@ QDateTime ExtractorPostprocessorPrivate::processTrainTripTime(QDateTime dt, cons
     } else if (station.identifier().startsWith(QLatin1String("ibnr:"))) {
         const auto record = KnowledgeDb::stationForIbnr(KnowledgeDb::IBNR{station.identifier().mid(5).toUInt()});
         tz = record.timezone.toQTimeZone();
+    } else if (!station.address().addressCountry().isEmpty()) {
+        tz = KnowledgeDb::timezoneForCountry(KnowledgeDb::CountryId{station.address().addressCountry()}).toQTimeZone();
     }
     if (!tz.isValid()) {
         return dt;
@@ -375,29 +392,87 @@ BusReservation ExtractorPostprocessorPrivate::processBusReservation(BusReservati
 
 BusTrip ExtractorPostprocessorPrivate::processBusTrip(BusTrip trip) const
 {
-    trip.setDepartureStation(processPlace(trip.departureStation()));
-    trip.setArrivalStation(processPlace(trip.arrivalStation()));
+    trip.setDepartureBusStop(processPlace(trip.departureBusStop()));
+    trip.setArrivalBusStop(processPlace(trip.arrivalBusStop()));
+    trip.setDepartureTime(processTimeForLocation(trip.departureTime(), trip.departureBusStop()));
+    trip.setArrivalTime(processTimeForLocation(trip.arrivalTime(), trip.arrivalBusStop()));
     return trip;
 }
 
 LodgingReservation ExtractorPostprocessorPrivate::processLodgingReservation(LodgingReservation res) const
 {
     res.setReservationFor(processPlace(res.reservationFor().value<LodgingBusiness>()));
+    res.setCheckinTime(processTimeForLocation(res.checkinTime(), res.reservationFor().value<LodgingBusiness>()));
+    res.setCheckoutTime(processTimeForLocation(res.checkoutTime(), res.reservationFor().value<LodgingBusiness>()));
     return processReservation(res);
 }
 
+TaxiReservation ExtractorPostprocessorPrivate::processTaxiReservation(TaxiReservation res) const
+{
+    res.setPickupLocation(processPlace(res.pickupLocation()));
+    res.setPickupTime(processTimeForLocation(res.pickupTime(), res.pickupLocation()));
+    return processReservation(res);
+}
+
+RentalCarReservation ExtractorPostprocessorPrivate::processRentalCarReservation(RentalCarReservation res) const
+{
+    res.setReservationFor(processRentalCar(res.reservationFor().value<RentalCar>()));
+    res.setPickupLocation(processPlace(res.pickupLocation()));
+    res.setDropoffLocation(processPlace(res.dropoffLocation()));
+    res.setPickupTime(processTimeForLocation(res.pickupTime(), res.pickupLocation()));
+    res.setDropoffTime(processTimeForLocation(res.dropoffTime(), res.dropoffLocation()));
+    return processReservation(res);
+}
+
+RentalCar ExtractorPostprocessorPrivate::processRentalCar(RentalCar car) const
+{
+    car.setName(car.name().trimmed());
+    return car;
+}
+
 FoodEstablishmentReservation ExtractorPostprocessorPrivate::processFoodEstablishmentReservation(FoodEstablishmentReservation res) const
 {
     res.setReservationFor(processPlace(res.reservationFor().value<FoodEstablishment>()));
+    res.setStartTime(processTimeForLocation(res.startTime(), res.reservationFor().value<FoodEstablishment>()));
+    res.setEndTime(processTimeForLocation(res.endTime(), res.reservationFor().value<FoodEstablishment>()));
     return processReservation(res);
 }
 
 TouristAttractionVisit ExtractorPostprocessorPrivate::processTouristAttractionVisit(TouristAttractionVisit visit) const
 {
     visit.setTouristAttraction(processPlace(visit.touristAttraction()));
+    visit.setArrivalTime(processTimeForLocation(visit.arrivalTime(), visit.touristAttraction()));
+    visit.setDepartureTime(processTimeForLocation(visit.departureTime(), visit.touristAttraction()));
     return visit;
 }
 
+EventReservation ExtractorPostprocessorPrivate::processEventReservation(EventReservation res) const
+{
+    res.setReservationFor(processEvent(res.reservationFor().value<Event>()));
+    return processReservation(res);
+}
+
+Event ExtractorPostprocessorPrivate::processEvent(Event event) const
+{
+    // normalize location to be a Place
+    if (JsonLd::isA<PostalAddress>(event.location())) {
+        Place place;
+        place.setAddress(event.location().value<PostalAddress>());
+        event.setLocation(place);
+    }
+
+    if (JsonLd::isA<Place>(event.location())) {
+        event.setLocation(processPlace(event.location().value<Place>()));
+
+        // try to obtain timezones if we have a location
+        event.setStartDate(processTimeForLocation(event.startDate(), event.location().value<Place>()));
+        event.setEndDate(processTimeForLocation(event.endDate(), event.location().value<Place>()));
+        event.setDoorTime(processTimeForLocation(event.doorTime(), event.location().value<Place>()));
+    }
+
+    return event;
+}
+
 template <typename T>
 T ExtractorPostprocessorPrivate::processReservation(T res) const
 {
@@ -406,6 +481,7 @@ T ExtractorPostprocessorPrivate::processReservation(T res) const
     return res;
 }
 
+
 Person ExtractorPostprocessorPrivate::processPerson(Person person) const
 {
     person.setName(person.name().simplified());
@@ -413,21 +489,37 @@ Person ExtractorPostprocessorPrivate::processPerson(Person person) const
     if (person.name().isEmpty() && !person.familyName().isEmpty() && !person.givenName().isEmpty()) {
         person.setName(person.givenName() + QLatin1Char(' ') + person.familyName());
     }
+
+    // strip prefixes, they break comparisons
+    static const char* honorificPrefixes[] = { "MR ", "MS ", "MRS " };
+    for (auto prefix : honorificPrefixes) {
+        if (person.name().startsWith(QLatin1String(prefix), Qt::CaseInsensitive)) {
+            person.setName(person.name().mid(strlen(prefix)));
+            break;
+        }
+    }
+
     return person;
 }
 
 template<typename T> T ExtractorPostprocessorPrivate::processPlace(T place) const
 {
-#ifdef HAVE_KCONTACTS
     auto addr = place.address();
-    if (!addr.addressCountry().isEmpty() && addr.addressCountry().size() != 2) {
+
+    // convert to ISO 3166-1 alpha-2 country codes
+    if (addr.addressCountry().size() > 2) {
         const auto isoCode = KContacts::Address::countryToISO(addr.addressCountry()).toUpper();
         if (!isoCode.isEmpty()) {
             addr.setAddressCountry(isoCode);
-            place.setAddress(addr);
         }
     }
-#endif
+
+    // upper case country codes
+    if (addr.addressCountry().size() == 2) {
+        addr.setAddressCountry(addr.addressCountry().toUpper());
+    }
+
+    place.setAddress(addr);
     return place;
 }
 
@@ -476,23 +568,62 @@ QVariantList ExtractorPostprocessorPrivate::processActions(QVariantList actions)
     return actions;
 }
 
+template <typename T>
+QDateTime ExtractorPostprocessorPrivate::processTimeForLocation(QDateTime dt, const T &place) const
+{
+    if (!dt.isValid() || dt.timeSpec() == Qt::TimeZone) {
+        return dt;
+    }
+
+    QTimeZone tz;
+    if (!place.address().addressCountry().isEmpty()) {
+        tz = KnowledgeDb::timezoneForCountry(KnowledgeDb::CountryId{place.address().addressCountry()}).toQTimeZone();
+    }
+    if (!tz.isValid()) {
+        return dt;
+    }
+
+    // prefer our timezone over externally provided UTC offset, if they match
+    if (dt.timeSpec() == Qt::OffsetFromUTC && tz.offsetFromUtc(dt) != dt.offsetFromUtc()) {
+        qCDebug(Log) << "UTC offset clashes with expected timezone!" << dt << dt.offsetFromUtc() << tz.id() << tz.offsetFromUtc(dt);
+        return dt;
+    }
+
+    if (dt.timeSpec() == Qt::OffsetFromUTC || dt.timeSpec() == Qt::LocalTime) {
+        dt.setTimeSpec(Qt::TimeZone);
+        dt.setTimeZone(tz);
+    } else if (dt.timeSpec() == Qt::UTC) {
+        dt = dt.toTimeZone(tz);
+    }
+    return dt;
+}
+
+
 bool ExtractorPostprocessorPrivate::filterReservation(const QVariant &res) const
 {
     if (JsonLd::isA<FlightReservation>(res)) {
         return filterFlight(res.value<FlightReservation>().reservationFor().value<Flight>());
     }
     if (JsonLd::isA<TrainReservation>(res)) {
-        return filterTrainOrBusTrip(res.value<TrainReservation>().reservationFor().value<TrainTrip>());
+        return filterTrainTrip(res.value<TrainReservation>().reservationFor().value<TrainTrip>());
     }
     if (JsonLd::isA<BusReservation>(res)) {
-        return filterTrainOrBusTrip(res.value<BusReservation>().reservationFor().value<BusTrip>());
+        return filterBusTrip(res.value<BusReservation>().reservationFor().value<BusTrip>());
     }
     if (JsonLd::isA<LodgingReservation>(res)) {
         return filterLodgingReservation(res.value<LodgingReservation>());
     }
+    if (JsonLd::isA<EventReservation>(res)) {
+        return filterEventReservation(res.value<EventReservation>());
+    }
+    if (JsonLd::isA<FoodEstablishmentReservation>(res)) {
+        return filterFoodReservation(res.value<FoodEstablishmentReservation>());
+    }
 
     // types without specific filters yet
-    if (JsonLd::isA<TouristAttractionVisit>(res) || JsonLd::isA<FoodEstablishmentReservation>(res)) {
+    if (JsonLd::isA<TouristAttractionVisit>(res) ||
+            JsonLd::isA<RentalCarReservation>(res) ||
+            JsonLd::isA<TaxiReservation>(res)) {
         return true;
     }
 
@@ -519,16 +650,33 @@ bool ExtractorPostprocessorPrivate::filterAirport(const Airport &airport) const
     return !airport.iataCode().isEmpty() || !airport.name().isEmpty();
 }
 
-template <typename T>
-bool ExtractorPostprocessorPrivate::filterTrainOrBusTrip(const T &trip) const
+bool ExtractorPostprocessorPrivate::filterTrainTrip(const TrainTrip &trip) const
 {
     return filterTrainOrBusStation(trip.departureStation())
            && filterTrainOrBusStation(trip.arrivalStation())
            && trip.departureTime().isValid() && trip.arrivalTime().isValid();
 }
 
+bool ExtractorPostprocessorPrivate::filterBusTrip(const BusTrip &trip) const
+{
+    return filterTrainOrBusStation(trip.departureBusStop())
+           && filterTrainOrBusStation(trip.arrivalBusStop())
+           && trip.departureTime().isValid() && trip.arrivalTime().isValid();
+}
+
 template <typename T>
 bool ExtractorPostprocessorPrivate::filterTrainOrBusStation(const T &station) const
 {
     return !station.name().isEmpty();
 }
+
+bool ExtractorPostprocessorPrivate::filterEventReservation(const EventReservation &res) const
+{
+    const auto event = res.reservationFor().value<Event>();
+    return !event.name().isEmpty() && event.startDate().isValid();
+}
+
+bool ExtractorPostprocessorPrivate::filterFoodReservation(const FoodEstablishmentReservation &res) const
+{
+    return res.startTime().isValid();
+}
