@@ -18,9 +18,11 @@
 #include "airportdbgenerator.h"
 #include "codegen.h"
 #include "wikidata.h"
+#include "../stringutil.h"
 
 #include <airportdb_p.h>
 
+#include <QDateTime>
 #include <QDebug>
 #include <QIODevice>
 #include <QJsonArray>
@@ -93,7 +95,7 @@ void AirportDbGenerator::merge(Airport &lhs, const Airport &rhs)
     } else if (rhs.coord.isValid()) {
         if (std::abs(lhs.coord.latitude - rhs.coord.latitude) > 0.2f || std::abs(lhs.coord.longitude - rhs.coord.longitude) > 0.2f) {
             ++m_coordinateConflicts;
-            qDebug() << lhs.uri << "has multiple conflicting coordinates";
+            qDebug() << lhs.label << lhs.iataCode << lhs.uri << "has multiple conflicting coordinates";
         }
         // pick always the same independent of the input order, so stabilize generated output
         lhs.coord.latitude = std::min(lhs.coord.latitude, rhs.coord.latitude);
@@ -105,7 +107,7 @@ bool AirportDbGenerator::fetchAirports()
 {
     // sorted by URI to stabilize the result in case of conflicts
     const auto airportArray = WikiData::query(R"(
-        SELECT DISTINCT ?airport ?airportLabel ?airportAltLabel ?iataCode ?icaoCode ?coord ?endDate ?demolished ?iataEndDate WHERE {
+        SELECT DISTINCT ?airport ?airportLabel ?airportAltLabel ?iataCode ?icaoCode ?coord ?endDate ?demolished ?openingDate ?iataEndDate WHERE {
             ?airport (wdt:P31/wdt:P279*) wd:Q1248784.
             ?airport p:P238 ?iataStmt.
             ?iataStmt ps:P238 ?iataCode.
@@ -113,6 +115,7 @@ bool AirportDbGenerator::fetchAirports()
             OPTIONAL { ?airport wdt:P625 ?coord. }
             OPTIONAL { ?airport wdt:P582 ?endDate. }
             OPTIONAL { ?airport wdt:P576 ?demolished. }
+            OPTIONAL { ?airport wdt:P1619 ?openingDate. }
             OPTIONAL { ?iataStmt pq:P582 ?iataEndDate. }
             SERVICE wikibase:label { bd:serviceParam wikibase:language "en". }
         } ORDER BY (?airport))", "wikidata_airports.json");
@@ -132,6 +135,12 @@ bool AirportDbGenerator::fetchAirports()
             continue;
         }
 
+        const auto openingdDt = QDateTime::fromString(obj.value(QLatin1String("openingDate")).toObject().value(QLatin1String("value")).toString(), Qt::ISODate);
+        if (openingdDt.isValid() && openingdDt > QDateTime::currentDateTime().addDays(120)) {
+            // skip future airports
+            continue;
+        }
+
         Airport a;
         a.uri = QUrl(obj.value(QLatin1String("airport")).toObject().value(QLatin1String("value")).toString());
         a.iataCode = obj.value(QLatin1String("iataCode")).toObject().value(QLatin1String("value")).toString();
@@ -203,11 +212,11 @@ void AirportDbGenerator::lookupTimezones()
 {
     for (auto it = m_airportMap.begin(); it != m_airportMap.end(); ++it) {
         if (!(*it).coord.isValid()) {
-            continue;
+            qDebug() << "Airport has no geo coordinate:" << (*it).label << (*it).iataCode << (*it).uri;
         }
-        (*it).tz = m_tzDb.timezoneForCoordinate((*it).coord);
+        (*it).tz = m_tzDb.timezoneForLocation((*it).country, (*it).coord);
         if ((*it).tz.isEmpty()) {
-            qDebug() << "Failed to find timezone for" << (*it).iataCode << (*it).label << (*it).coord.latitude << (*it).coord.longitude << (*it).uri;
+            qDebug() << "Failed to find timezone for" << (*it).iataCode << (*it).label << (*it).country << (*it).coord.latitude << (*it).coord.longitude << (*it).uri;
             ++m_timezoneLoopupFails;
             continue;
         }
@@ -217,7 +226,7 @@ void AirportDbGenerator::lookupTimezones()
 void KItinerary::Generator::AirportDbGenerator::indexNames()
 {
     for (auto it = m_airportMap.begin(); it != m_airportMap.end(); ++it) {
-        auto l = QString(it.value().label + QLatin1Char(' ') + it.value().alias)
+        auto l = StringUtil::normalize(it.value().label + QLatin1Char(' ') + it.value().alias)
                  .split(QRegularExpression(QStringLiteral("[ 0-9/'\"\\(\\)&\\,.–„-]")), QString::SkipEmptyParts);
         std::for_each(l.begin(), l.end(), [](QString &s) {
             s = s.toCaseFolded();
@@ -260,6 +269,7 @@ bool AirportDbGenerator::generate(QIODevice* out)
 #include "airportdb_p.h"
 #include "knowledgedb.h"
 #include "timezonedb.h"
+#include "timezonedb_data_p.h"
 
 #include <limits>
 
@@ -269,7 +279,7 @@ namespace KItinerary {
 namespace KnowledgeDb {
 
 // airport data sorted by IATA code
-// the corresponding index is used to acces data the following tables
+// the corresponding index is used to access data the following tables
 static constexpr Airport airport_table[] = {
 )");
 
@@ -280,7 +290,7 @@ static constexpr Airport airport_table[] = {
         out->write("\"}, ");
         CodeGen::writeCountryIsoCode(out, m_airportMap.value(it.value()).country);
         out->write(", ");
-        CodeGen::writeTimezone(out, &m_tzDb, m_airportMap.value(it.value()).tz);
+        CodeGen::writeTimezone(out, m_airportMap.value(it.value()).tz);
         out->write("}, // ");
         out->write(m_airportMap.value(it.value()).label.toUtf8());
         out->write("\n");
